@@ -13,9 +13,17 @@ return (new PhpCsFixer\Config())
         'heredoc_to_nowdoc' => false,
         'ordered_imports' => true,
         'phpdoc_types_order' => ['null_adjustment' => 'always_last', 'sort_algorithm' => 'none'],
-        // TODO: Remove once the "compiler_optimized" set includes "sprintf"
-        'native_function_invocation' => ['include' => ['@compiler_optimized', 'sprintf'], 'scope' => 'all'],
-        ])
+        'header_comment' => [
+            'header' => <<<EOF
+                This file is part of Twig.
+
+                (c) Fabien Potencier
+
+                For the full copyright and license information, please view the LICENSE
+                file that was distributed with this source code.
+                EOF
+        ],
+    ])
     ->setRiskyAllowed(true)
     ->setFinder((new PhpCsFixer\Finder())->in(__DIR__))
 ;
