@@ -33,11 +33,11 @@ class FileExtensionEscapingStrategy
      */
     public static function guess(string $name)
     {
-        if (\in_array(substr($name, -1), ['/', '\\'])) {
+        if (\in_array(substr($name, -1), ['/', '\\'], true)) {
             return 'html'; // return html for directories
         }
 
-        if ('.twig' === substr($name, -5)) {
+        if (str_ends_with($name, '.twig')) {
             $name = substr($name, 0, -5);
         }
 
@@ -45,6 +45,7 @@ class FileExtensionEscapingStrategy
 
         switch ($extension) {
             case 'js':
+            case 'json':
                 return 'js';
 
             case 'css':
