@@ -14,37 +14,49 @@ namespace Twig\Node\Expression;
 
 use Twig\Compiler;
 use Twig\Extension\SandboxExtension;
+use Twig\Node\Expression\Variable\ContextVariable;
 use Twig\Template;
 
-class GetAttrExpression extends AbstractExpression
+class GetAttrExpression extends AbstractExpression implements SupportDefinedTestInterface
 {
+    use SupportDefinedTestDeprecationTrait;
+    use SupportDefinedTestTrait;
+
     /**
      * @param ArrayExpression|NameExpression|null $arguments
      */
-    public function __construct(AbstractExpression $node, AbstractExpression $attribute, ?AbstractExpression $arguments, string $type, int $lineno)
+    public function __construct(AbstractExpression $node, AbstractExpression $attribute, ?AbstractExpression $arguments, string $type, int $lineno, bool $nullSafe = false)
     {
         $nodes = ['node' => $node, 'attribute' => $attribute];
         if (null !== $arguments) {
             $nodes['arguments'] = $arguments;
         }
 
-        if ($arguments && !$arguments instanceof ArrayExpression && !$arguments instanceof NameExpression) {
+        if ($arguments && !$arguments instanceof ArrayExpression && !$arguments instanceof ContextVariable) {
             trigger_deprecation('twig/twig', '3.15', \sprintf('Not passing a "%s" instance as the "arguments" argument of the "%s" constructor is deprecated ("%s" given).', ArrayExpression::class, static::class, $arguments::class));
         }
 
-        parent::__construct($nodes, ['type' => $type, 'is_defined_test' => false, 'ignore_strict_check' => false, 'optimizable' => true], $lineno);
+        parent::__construct($nodes, ['type' => $type, 'ignore_strict_check' => false, 'optimizable' => !$nullSafe, 'null_safe' => $nullSafe], $lineno);
+    }
+
+    public function enableDefinedTest(): void
+    {
+        $this->definedTest = true;
+        $this->changeIgnoreStrictCheck($this);
     }
 
     public function compile(Compiler $compiler): void
     {
         $env = $compiler->getEnvironment();
         $arrayAccessSandbox = false;
+        $nullSafe = $this->getAttribute('null_safe');
+        $objectVar = null;
 
         // optimize array calls
         if (
             $this->getAttribute('optimizable')
             && (!$env->isStrictVariables() || $this->getAttribute('ignore_strict_check'))
-            && !$this->getAttribute('is_defined_test')
+            && !$this->definedTest
             && Template::ARRAY_CALL === $this->getAttribute('type')
         ) {
             $var = '$'.$compiler->getVarName();
@@ -83,14 +95,27 @@ class GetAttrExpression extends AbstractExpression
             ;
         }
 
-        $compiler->raw('CoreExtension::getAttribute($this->env, $this->source, ');
-
         if ($this->getAttribute('ignore_strict_check')) {
             $this->getNode('node')->setAttribute('ignore_strict_check', true);
         }
 
+        if ($nullSafe) {
+            $objectVar = '$'.$compiler->getVarName();
+            $compiler
+                ->raw('((null === ('.$objectVar.' = ')
+                ->subcompile($this->getNode('node'))
+                ->raw(')) ? null : ');
+        }
+
+        $compiler->raw('CoreExtension::getAttribute($this->env, $this->source, ');
+
+        if ($nullSafe) {
+            $compiler->raw($objectVar);
+        } else {
+            $compiler->subcompile($this->getNode('node'));
+        }
+
         $compiler
-            ->subcompile($this->getNode('node'))
             ->raw(', ')
             ->subcompile($this->getNode('attribute'))
         ;
@@ -103,7 +128,7 @@ class GetAttrExpression extends AbstractExpression
 
         $compiler->raw(', ')
             ->repr($this->getAttribute('type'))
-            ->raw(', ')->repr($this->getAttribute('is_defined_test'))
+            ->raw(', ')->repr($this->definedTest)
             ->raw(', ')->repr($this->getAttribute('ignore_strict_check'))
             ->raw(', ')->repr($env->hasExtension(SandboxExtension::class))
             ->raw(', ')->repr($this->getNode('node')->getTemplateLine())
@@ -113,5 +138,19 @@ class GetAttrExpression extends AbstractExpression
         if ($arrayAccessSandbox) {
             $compiler->raw(')');
         }
+
+        if ($nullSafe) {
+            $compiler->raw(')');
+        }
+    }
+
+    private function changeIgnoreStrictCheck(self $node): void
+    {
+        $node->setAttribute('optimizable', false);
+        $node->setAttribute('ignore_strict_check', true);
+
+        if ($node->getNode('node') instanceof self) {
+            $this->changeIgnoreStrictCheck($node->getNode('node'));
+        }
     }
 }
